import Image
import ImageFile
import sys
import getopt
import os
import stat
import string

ImageFile.MAXBLOCK = 1000000 # default is 64k

def resize(fullpath, thumbpath, maxwh, qual, verbose):
    """Resizes an image
    fullPath  - source image
    thumbPath - destination image
    maxwh     - maximum width / height of the image in pixels
    qual      - quality of the resized imasge
    verbose   - print trace of operations
    returns dictionary containing source and destionation paths and sizes:"""
    updateThumb = True
    if os.path.exists(thumbpath):
        thumbstat   = os.stat(thumbpath)
        filestat    = os.stat(fullpath)
        updateThumb = filestat.st_mtime > thumbstat.st_mtime

    try:
        im = Image.open(fullpath) 
    except:
        print 'Unable to open ' + fullpath
        return None

    try:
        im.load()
    except Exception, e:
        print 'Error loading ' + f + ' ' + str(e)
        return None

    thumbdir = os.path.split(thumbpath)[0]
    if not os.path.exists(thumbdir):
        try:
            os.mkdir(thumbdir)
        except:
            print 'unable to create dir ' + thumbdir
            return None

    if im.size[0] > im.size[1]:
        size = (maxwh, int((float(im.size[1]) / 
                            float(im.size[0])) * float(maxwh)))
    else:
        size = (int((float(im.size[0]) / 
                     float(im.size[1])) * float(maxwh)), maxwh)

    if updateThumb:
        if verbose:
            print str(os.path.split(fullpath)[1] + ' ' + 
                      str(im.size) + ' --> ' + str(size))

        try:
            im2  = im.resize(size, Image.ANTIALIAS)
            try:
                im2.save(thumbpath, quality=qual, optimize=1)
            except:
                try:
                    im2.save(thumbpath, quality=qual)
                except:
                    try:
                        im2.save(thumbpath)
                    except:
                        return None
        except:
            return None

    return {'src'    : fullpath,
            'srcwh'  : im.size,
            'dest'   : thumbpath,
            'destwh' : size}


def addRemoveThumbs(path, maxwh, add, thumbs, qual, 
                    ignoreDirs, verbose, prefix):
    # don't thumbnail thumbs dir or ignore dirs
    if thumbs in path:
        return
        
    for d in ignoreDirs:
        if d in path:
            return

    files       = os.listdir(path)
    removeFiles = []
    for f in files:
        fullpath = os.path.join(path, f)
        filestat = os.stat(fullpath)
        filemode = filestat[stat.ST_MODE]
        if stat.S_ISDIR(filemode):
           addRemoveThumbs(fullpath, maxwh, add, thumbs, 
                           qual, ignoreDirs, verbose)
           
        elif stat.S_ISREG(filemode):
            thumbdir  = os.path.join(path, thumbs)
            thumbpath = os.path.join(thumbdir, prefix + f)
            if add:
                resize(fullpath, thumbpath, maxwh, qual, verbose)
            else:
                removeFiles.append(thumbpath)

    for f in removeFiles:
        try:
            os.remove(f)
            if verbose:
                print 'rm ' + f
        except:
            pass
    
    if len(removeFiles):
        try:
            os.rmdir(thumbdir)
        except:
            pass

def main():
    maxwh      = 120        # max size of thumbs
    add        = True       # add thumbs
    thumbsDir  = '__thumbs' # thumb dir
    ignoreDirs = []         # dirs to not create thumbs for
    verbose    = False      # spew info
    quality    = 85         # jpeg quality
    prefix     = ''         # thumbnail prefix
    
    opts, args = getopt.getopt(sys.argv[1:], 'hrm:t:q:i:vp:', 
        ['help', 'remove', 'maxwh=', 'thumbsdir=', 
         'quality=', 'ignore=', 'verbose', 'prefix='])
    for o, v in opts:
        if o == '--remove' or o == '-r':
            add = False
        elif o == '--maxwh' or o == '-m':
            _maxwh  = int(v)
            if _maxwh:
                maxwh = _maxwh
        elif o == '--thumbsdir' or o == '-t':
            thumbsDir = v
        elif o == '--quality' or o == '-q':
            q = int(v)
            if q:
                quality = q
        elif o == '--ignore' or o == '-i':
            for t in string.split(v, '|'):
                ignoreDirs.append(t)
        elif o == '--verbose' or o == '-v':
            verbose = True
        elif o == '--prefix' or o == '-p':
            prefix  = v

    addRemoveThumbs(os.getcwd(), maxwh, add, thumbsDir, 
                    quality, ignoreDirs, verbose, prefix)

if __name__ == '__main__':
    main()
